@@ -6,7 +6,7 @@ use Carp;
 
 use Graph::Layout::Aesthetic::Force;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 require XSLoader;
 XSLoader::load('Graph::Layout::Aesthetic', $VERSION);
@@ -37,8 +37,7 @@ sub gloss {
     my $iter = delete $params{iterations};
     $iter = 1e3 if !defined $iter;
     croak "Iterations should be >= 0" if $iter < 0;
-    croak "Iterations should be an integer" if
-        $iter != int($iter);
+    croak "Iterations should be an integer" if $iter != int($iter);
 
     my $mon_delay = delete $params{monitor_delay};
     $mon_delay = 2 if !defined $mon_delay;
@@ -80,18 +79,24 @@ sub pause {
 sub coordinates_to_graph {
     my ($aglo, $graph, %params) = @_;
 
-    my $pos = delete $params{pos_attribute};
-    $pos  = "pos"  unless defined $pos;
+    my $pos = exists $params{pos_attribute} ? 
+        delete $params{pos_attribute} : "layout_pos";
+
+    my $min_attr = exists $params{min_attribute} ? 
+        delete $params{min_attribute} : "layout_min";
+
+    my $max_attr = exists $params{max_attribute} ? 
+        delete $params{max_attribute} : "layout_max";
 
     my $name = delete $params{id_attribute};
-    $name = "aglo" unless defined $name;
+    $name = "layout_id" unless defined $name;
 
     croak "Unknown parameter ", join(", ", keys %params) if %params;
 
-    my $coordinates = $aglo->all_coordinates;
     my $ref_name = ref($name) && 1;
     my $id;
     if (ref $pos) {
+        my $coordinates = $aglo->all_coordinates;
         my @pos = @$pos;
         @pos == $aglo->nr_dimensions || croak "Number of entries in the position attribute array must be equal to the number of dimensions";
         for my $vertex ($ref_name ? keys %$name : $graph->vertices) {
@@ -102,7 +107,8 @@ sub coordinates_to_graph {
             $graph->set_attribute($pos[$_], $vertex, $coordinate->[$_]) for
                 0..$#pos;
         }
-    } else {
+    } elsif (defined($pos)) {
+        my $coordinates = $aglo->all_coordinates;
         for my $vertex ($ref_name ? keys %$name : $graph->vertices) {
             defined($id = $ref_name ? $name->{$vertex} :
                     $graph->get_attribute($name, $vertex)) ||
@@ -110,6 +116,24 @@ sub coordinates_to_graph {
             $graph->set_attribute($pos, $vertex, $coordinates->[$id]);
         }
     }
+
+    if (defined $min_attr || defined $max_attr) {
+        my ($min, $max) = $aglo->frame;
+
+        if (ref $min_attr) {
+            @$min_attr == $aglo->nr_dimensions || croak "Number of entries in the minimum attribute array must be equal to the number of dimensions";
+            $graph->set_attribute($_, shift @$min) for @$min_attr;
+        } elsif (defined $min_attr) {
+            $graph->set_attribute($min_attr, $min);
+        }
+
+        if (ref $max_attr) {
+            @$max_attr == $aglo->nr_dimensions || croak "Number of entries in the maximum attribute array must be equal to the number of dimensions";
+            $graph->set_attribute($_, shift @$max) for @$max_attr;
+        } elsif (defined $max_attr) {
+            $graph->set_attribute($max_attr, $max);
+        }
+    }
 }
 
 sub gloss_graph {
@@ -121,11 +145,15 @@ sub gloss_graph {
     $dim = 2 unless defined($dim);
 
     my $pos = delete $params{pos_attribute};
-    $pos = "pos" unless defined $pos;
+    $pos = "layout_pos" unless defined $pos;
 
     defined(my $forces = delete $params{forces}) ||
         croak "No forces were defined";
 
+    my @to_graph_params;
+    exists $params{$_} && push @to_graph_params, $_, delete $params{$_} for
+        qw(min_attribute max_attribute);
+
     require Graph::Layout::Aesthetic::Topology;
     my $topology = Graph::Layout::Aesthetic::Topology->from_graph
         ($graph,
@@ -153,8 +181,9 @@ sub gloss_graph {
     # This will implicitely check for bad parameters
     $aglo->gloss(%params);
     $aglo->coordinates_to_graph($graph,
+                                id_attribute  => \%id,
                                 pos_attribute => $pos,
-                                id_attribute  => \%id)
+                                @to_graph_params);
 }
 
 1;
@@ -216,10 +245,14 @@ Graph::Layout::Aesthetic - A module for laying out graphs
 
   $aglo->coordinates_to_graph($graph, %parameters);
   # Valid parameter keys/value pairs are:
-  #   pos_attribute => $string
-  #   pos_attribute => \@strings
   #   id_attribute  => $string
   #   id_attribute  => \%name_to_num
+  #   pos_attribute => $string
+  #   pos_attribute => \@strings
+  #   min_attribute => $string
+  #   min_attribute => \@strings
+  #   max_attribute => $string
+  #   max_attribute => \@strings
 
   Graph::Layout::Aesthetic->gloss_graph($graph, %parameters)
   # Valid parameter keys/value pairs are:
@@ -286,7 +319,7 @@ The package also comes with a simple commandline tool L<gloss.pl|gloss.pl(1)>
                                             node_repulsion  => 1,
                                             min_edge_length => 1
                                         });
-  # That's all folks. Vertex positions will be in the "pos" attribute
+  # That's all folks. Vertex positions will be in the "layout_pos" attribute
 
 =head1 METHODS
 
@@ -636,39 +669,79 @@ this call.
 
 =item X<coordinates_to_graph>$aglo->coordinates_to_graph($graph, %parameters)
 
-Copies the current state coordinates of $aglo to the standard L<Graph|Graph>
-object $graph.
+Copies the current state coordinates of $aglo to vertex attributes of the 
+standard L<Graph|Graph> object $graph. It also stores
+L<the containing frame|"frame"> as graph attributes.
 
 %parameters are key/value pairs. Recognized are:
 
 =over
 
-=item pos_attribute => $name
-
-$name is the graph vertices attribute that will be used to set the
-coordinates and defaults to C<"pos">. If this attribute is a plain string,
-its attribute value will become an array reference to the coordinates of that
-vertex. If this attribute is an array of names (size equal to the
-L<number of dimensions of the layout space|"nr_dimensions">), then it
-will set an attribute for each name whose value will be the corresponding
-coordinate. So if for example you want your $graph nodes to have an attribute
-x for the first coordinate and y for the second you could use:
-
-    $aglo->coordinates_to_graph($graph, ["x", "y"]);
-    # And now you can get the second coordinate of vertex foo by doing:
-    my $y = $graph->get_attribute("y", "foo");
-
-=item pos_attribute => $name
+=item X<coordinates_to_graph_id_attribute>id_attribute => $name
 
 $name is the the graph vertices attribute that for each vertex identifies the
 corresponding vertex in L<$aglo->topology|"topology">, and defaults to
-C<"aglo">. It may also be a hash reference, in which case node ids are looked
-up in the hash instead of as atrributes in the graph. So this argument is
-compatible with the
+C<"layout_id">. It may also be a hash reference, in which case node ids are 
+looked up in the hash instead of as atrributes in the graph. So this argument 
+is compatible with the
 L<id_attribute parameter|Graph::Layout::Aesthetic::Topology/from_graph_attribute>
 of the
 L<Graph::Layout::Aesthetic::Topology from_graph method|Graph::Layout::Aesthetic::Topology/from_graph>.
 
+=item X<coordinates_to_graph_pos_attribute>pos_attribute => $name
+
+$name is the graph vertices attribute that will be used to set the
+coordinates and defaults to C<"layout_pos"> if not given at all. 
+
+If $name is undef, no vertex attribute will be set.
+
+If $name is a plain string, it will set an attribute with this name on each 
+vertex. The value will be an array reference to the coordinates of that vertex.
+
+If $name is an array of names (size equal to the
+L<number of dimensions of the layout space|"nr_dimensions">), then for each 
+vertex it will set an attribute for each element in $name whose value will be 
+the corresponding coordinate component. So if for example you want your $graph 
+nodes to have an attribute C<"layout_pos1"> for the first coordinate and 
+C<"layout_pos2"> for the second (this is what 
+L<Graph::Layouter|Graph::Layouter> uses and 
+L<Graph::Renderer|Graph::Renderer> expects) you could use:
+
+  $aglo->coordinates_to_graph($graph, 
+                              pos_attribute => ["layout_pos1", "layout_pos2"]);
+  # And now you can get the second coordinate of vertex foo by doing:
+  my $y = $graph->get_attribute("layout_pos2", "foo");
+
+=item X<coordinates_to_graph_min_attribute>min_attribute => $name
+
+$name is the graph attribute that will be used to set the minimum coordinates
+for the frame containing all poins (see L<the frame method|"frame"> and 
+defaults to C<"layout_min"> if not given at all. 
+
+Like L<pos_attribute|"coordinates_to_graph_pos_attribute"> you can give it
+an undef value (attribute will not be set), a string (one attribute will be
+set) or an array reference (an attribute will be set for each string in the 
+array).
+
+=item X<coordinates_to_graph_max_attribute>max_attribute => $name
+
+$name is the graph attribute that will be used to set the maximum coordinates
+for the frame containing all poins (see L<the frame method|"frame"> and 
+defaults to C<"layout_max"> if not given at all. 
+
+Like L<pos_attribute|"coordinates_to_graph_pos_attribute"> you can give it
+an undef value (attribute will not be set), a string (one attribute will be
+set) or an array reference (an attribute will be set for each string in the 
+array).
+
+So for complete compatibility with L<Graph::Layouter|Graph::Layouter> and 
+L<Graph::Renderer|Graph::Renderer> you can use:
+
+  $aglo->coordinates_to_graph($graph, 
+                              pos_attribute => ["layout_pos1", "layout_pos2"],
+                              min_attribute => ["layout_min1", "layout_min2"],
+                              max_attribute => ["layout_max1", "layout_max2"]);
+
 =back
 
 =item X<gloss_graph>Graph::Layout::Aesthetic->gloss_graph($graph, %parameters)
@@ -762,10 +835,20 @@ L<monitor_delay parameter |"gloss_monitor_delay">
 
 The same as the L<coordinates_to_graph|"coordinates_to_graph">
 L<pos_attribute parameter|"coordinates_to_graph_pos_attribute">
-(and also defaults to C<"pos">.
+(and also defaults to C<"layout_pos">.
 It's also the default attribute initial coordinates come from
 if L<hold|gloss_graph_hold> is 1.
 
+=item X<gloss_graph_graph_min_attribute>min_attribute => $name
+
+The same as the L<coordinates_to_graph|"coordinates_to_graph">
+L<min_attribute parameter|"coordinates_to_graph_min_attribute">.
+
+=item X<gloss_graph_graph_max_attribute>max_attribute => $name
+
+The same as the L<coordinates_to_graph|"coordinates_to_graph">
+L<max_attribute parameter|"coordinates_to_graph_max_attribute">.
+
 =back
 
 =back
@@ -779,7 +862,10 @@ None.
 L<http://www.cs.ucla.edu/~stott/aglo/>,
 L<Graph::Layout::Aesthetic::Topology>,
 L<Graph::Layout::Aesthetic::Monitor::GnuPlot>,
-L<Graph>
+L<Graph>,
+L<Graph::Layouter>,
+L<Graph::Renderer>,
+L<GraphViz>
 
 =head1 AUTHOR
 
